#include "StdAfx.h"
#include "MemoryPool.h"

CMemoryPool::CMemoryPool(size_t PoolSize, size_t MemorySize)
{
	for(size_t i=0; i<PoolSize; ++i) 
	{
		MEMORY_DESC MD;

		MD.bUsing = false;
		MD.pBuffer = new BYTE [MemorySize];
		MD.Index = i;

		pool_.push_back(MD);
	}

	freeMemIndex_ = 0;
	hEvent_ = ::CreateEvent(NULL, TRUE, TRUE, NULL);
}

CMemoryPool::~CMemoryPool(void)
{
	::CloseHandle(hEvent_);

	for(size_t i=0; i<pool_.size(); ++i) 
	{
		delete [] pool_[i].pBuffer;
	}
}

CMemoryPool::MEMORY_DESC *CMemoryPool::Take()
{
	WaitForSingleObject(hEvent_, INFINITE);

	scs_.Lock();

	if(!pool_[freeMemIndex_].bUsing) {
		pool_[freeMemIndex_].bUsing = true;
		int oldMemIndex = freeMemIndex_;
		freeMemIndex_++;

		if(freeMemIndex_>(pool_.size()-1)) {
			ResetEvent(hEvent_);
			freeMemIndex_ = 0;
		}

		/*///////////////////////////////////
		if(cs_.IsVisible()) {
			char szTmpStr[127];
			szTmpStr[0] = '\t';
			szTmpStr[1] = '[';
			for(size_t j=0; j<pool_.size(); ++j) {
				if(pool_[j].bUsing) 
					szTmpStr[j+2] = '1';
				else 
					szTmpStr[j+2] = '0';
			}
			szTmpStr[pool_.size()+2] = ']';
			szTmpStr[pool_.size()+3] = '\0';
			cs_.Writeln(szTmpStr, 0,1,0);
		}
		///////////////////////////////////*/

		scs_.Unlock();

		return &pool_[oldMemIndex];
	}

	ResetEvent(hEvent_);
	scs_.Unlock();

	return Take();
}

void CMemoryPool::Release(MEMORY_DESC *pMD)
{
	scs_.Lock();

	pMD->bUsing = false;
	freeMemIndex_ = pMD->Index;
	::SetEvent(hEvent_);

	scs_.Unlock();
}